//
//  kmpg
//
//  Copyright (C) 1997 Bjarni Ivarsson
//  email: Bjarni.Ivarsson@swipnet.se
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <iostream.h>

#include "playlist.h"

#include <qpushbt.h>
#include <qpainter.h>
#include <qregexp.h>

#include <kiconloader.h>
#include <kconfig.h>
#include <kwm.h>

QString extractFilename( QString _path, const char *_regexp )
{
   QRegExp r = QRegExp( _regexp, FALSE, FALSE );
   QString string;
   int i;
   
   string = _path;
      i = string.findRev( '/' );
   if( i!=-1 )
      string = string.right( string.length() - i - 1 );
   i = string.findRev( r );
   if( i!=-1 )
      string.remove( i, string.length() );

   return string;
}

PlaylistItem::PlaylistItem( KURL url )
{
   _url = url;
   setText( extractFilename( url.path(), ".mp[123]" ) );
   _played = FALSE;
}

void PlaylistItem::paint( QPainter *p )
{
   QFont font = p->font();
   
   font.setBold( !_played );
   QFontMetrics fm(font);
   int ypos = fm.ascent() + fm.leading()/2;
   p->save();
   p->setFont(font);
   p->drawText( 5, ypos, text() );
   p->restore();   
}

int PlaylistItem::height( const QListBox *lb ) const
{
   return lb->fontMetrics().lineSpacing()+2;
}

int PlaylistItem::width( const QListBox *lb ) const
{
   return lb->fontMetrics().width( text() )+6;
}

PlaylistBox::PlaylistBox( QWidget * parent=0, const char * name=0, WFlags f=0 )
   : QListBox( parent, name, f )
{
}

void PlaylistBox::doClear()
{
   clear();
}

void PlaylistBox::doInsert( int index, QString file )
{
   PlaylistItem *item = new PlaylistItem( KURL(file) );
   insertItem( item, index );
}

void PlaylistBox::doUpdate( int index, QString file )
{
   removeItem( index );
   doInsert( index, file );
}

void PlaylistBox::doRemove( int index )
{
   removeItem( index );
}

void PlaylistBox::doMoveUp( int index )
{
}

void PlaylistBox::doMoveDown( int index )
{
}

void PlaylistBox::doSetPlayed( int index, bool pl )
{
   PlaylistItem *it = (PlaylistItem*)item(index);
   
   it->setPlayed( pl );
   repaint();
}

void PlaylistBox::doSetAllPlayed( bool pl )
{
   int i;
   
   for( i=0; i<count(); i++ )
      doSetPlayed( i, pl );
}

void PlaylistBox::doTrackSelected( int track )
{
   setCurrentItem( track );
}

#define CAPTION(file) QString( klocale->translate("Playlist Editor: ") )+file

Playlist::Playlist( PlaylistMgr *mgr ) :
  QDialog()
{
   KDNDDropZone *dropZone = new KDNDDropZone( this, DndURL );
   connect( dropZone, SIGNAL( dropAction( KDNDDropZone* )),
	    SLOT( slotDrop( KDNDDropZone*)));

   listbox = new PlaylistBox( this, "Playlist");
   setCaption( CAPTION("<noname>") );
   
   buttonPanel = new QWidget( this );
   
   QPushButton *pb1 = new QPushButton( klocale->translate("Close"), buttonPanel );
   pb1->setGeometry( 10, 5, 80, 25 );
   QPushButton *pb2 = new QPushButton( klocale->translate("Load playlist"), buttonPanel );
   pb2->setGeometry( 10, 40, 80, 25 );
   QPushButton *pb3 = new QPushButton( klocale->translate("Save playlist"), buttonPanel );
   pb3->setGeometry( 10, 65, 80, 25 );
   QPushButton *pb4 = new QPushButton( klocale->translate("Add playlist"), buttonPanel );
   pb4->setGeometry( 10, 100, 80, 25 );
   QPushButton *pb5 = new QPushButton( klocale->translate("Add MPEG"), buttonPanel );
   pb5->setGeometry( 10, 125, 80, 25 );
   QPushButton *pb6 = new QPushButton( "up", buttonPanel );
   pb6->setGeometry( 10, 160, 80, 12 );
   QPushButton *pb7 = new QPushButton( "dn", buttonPanel );
   pb7->setGeometry( 10, 172, 80, 12 );
   QPushButton *pb8 = new QPushButton( klocale->translate("Del MPEG"), buttonPanel );
   pb8->setGeometry( 10, 194, 80, 25 );
   QPushButton *pb9 = new QPushButton( klocale->translate("Clear list"), buttonPanel );
   pb9->setGeometry( 10, 219, 80, 25 );

   QPixmap pix;
   KIconLoader *loader = KApplication::getKApplication()->getIconLoader();
   pix = loader->loadIcon("up.xpm");
   pb6->setPixmap( pix );
   pix = loader->loadIcon("down.xpm");
   pb7->setPixmap( pix );

   playlistMgr = mgr;

   connect( listbox, SIGNAL(selected(int)), 
	    playlistMgr, SLOT(playTrack(int)) );
   connect( playlistMgr, SIGNAL(sigClear()),
	    listbox, SLOT(doClear()) );
   connect( playlistMgr, SIGNAL(sigInsert( int, QString )),
	    listbox, SLOT(doInsert( int, QString )) );
   connect( playlistMgr, SIGNAL(sigUpdate( int, QString )),
	    listbox, SLOT(doUpdate( int, QString )) );
   connect( playlistMgr, SIGNAL(sigRemove( int )),
	    listbox, SLOT(doRemove( int )) );
   connect( playlistMgr, SIGNAL(sigMoveUp( int )),
	    listbox, SLOT(doMoveUp( int )) );
   connect( playlistMgr, SIGNAL(sigMoveDown( int )),
	    listbox, SLOT(doMoveDown( int )) );
   connect( playlistMgr, SIGNAL(sigSetPlayed( int, bool )),
	    listbox, SLOT(doSetPlayed( int, bool )) );   
   connect( playlistMgr, SIGNAL(sigSetAllPlayed( bool )),
	    listbox, SLOT(doSetAllPlayed( bool )) );
   connect( playlistMgr, SIGNAL(sigTrackSelected( int )),
	    listbox, SLOT(doTrackSelected( int )) );

   connect( pb1, SIGNAL(clicked()), SLOT(slotClose()) );
   connect( pb2, SIGNAL(clicked()), SLOT(slotLoadPlaylist()) );
   connect( pb3, SIGNAL(clicked()), SLOT(slotSavePlaylist()) );
   connect( pb4, SIGNAL(clicked()), SLOT(slotAddPlaylist()) );
   connect( pb5, SIGNAL(clicked()), SLOT(slotAddMPEG()) );
   connect( pb6, SIGNAL(clicked()), SLOT(slotUp()) );
   connect( pb7, SIGNAL(clicked()), SLOT(slotDown()) );
   connect( pb8, SIGNAL(clicked()), SLOT(slotDelMPEG()) );
   connect( pb9, SIGNAL(clicked()), playlistMgr, SLOT(clear()) );
   
   setMinimumSize( 350, 250 );
   resize( 350, 250 ); // Use the smallest size by default
   KWM::setWmCommand( handle(), "" ); // Disable SM for this window
}

Playlist::~Playlist()
{
//   cerr << "Playlist::~Playlist()" << endl;
}

void Playlist::readProperties( KConfig *config )
{
   int _x, _y, _w, _h;

   _x = config->readNumEntry( "PlaylistXPos", 10 );
   _y = config->readNumEntry( "PlaylistYPos", 10 );
   _w = config->readNumEntry( "PlaylistWitdh", 350 );
   _h = config->readNumEntry( "PlaylistHeight", 250 );
   setGeometry( _x, _y, _w, _h );
   if( config->readNumEntry( "PlaylistVisible", 0 ) )
      show();
}

void Playlist::saveProperties( KConfig *config )
{
   config->writeEntry( "PlaylistXPos", geometry().x() );
   config->writeEntry( "PlaylistYPos", geometry().y() );
   config->writeEntry( "PlaylistWidth", geometry().width() );
   config->writeEntry( "PlaylistHeight", geometry().height() );
   config->writeEntry( "PlaylistVisible", isVisible() );
}

void Playlist::resizeEvent( QResizeEvent* )
{
   listbox->setGeometry( 5, 5, width()-105, height()-10 );
   buttonPanel->setGeometry( width()-100, 0, 100, height() );
}

void Playlist::slotShow()
{
   show();
}

void Playlist::slotClose()
{
   hide();
}

void Playlist::slotLoadPlaylist()
{
   QString filename;

   filename = QFileDialog::getOpenFileName( 0, "*.m3u" );
   if( filename.isNull() )
      return;

   playlistMgr->load( KURL( filename ) );
   setCaption( CAPTION(extractFilename( filename, ".m[123]u" )) );
}

void Playlist::slotSavePlaylist()
{
   QString filename;
   
   filename = QFileDialog::getSaveFileName( 0, "*.m3u" );
   if( filename.isNull() )
      return;
   
   playlistMgr->save( KURL( filename ) );
   setCaption( CAPTION(extractFilename( filename, ".m[123]u" )) );
}

void Playlist::slotAddPlaylist()
{
   QString filename;

   filename = QFileDialog::getOpenFileName( 0, "*.m3u" );
   if( filename.isNull() )
      return;

   playlistMgr->add( KURL( filename ) );
}

void Playlist::slotAddMPEG()
{
   QString filename;

   filename = QFileDialog::getOpenFileName( 0, "*.mp?" );
   if( filename.isNull() )
      return;
   playlistMgr->insert( KURL( filename.data() ) );
}

void Playlist::slotUp()
{
    int _item = listbox->currentItem();
   
    if( _item < 1 )
       return;

    listbox->setAutoUpdate( FALSE );
    playlistMgr->moveUp( _item );
    listbox->setAutoUpdate( TRUE );
    listbox->repaint();
}

void Playlist::slotDown()
{
    int _item = listbox->currentItem();
   
    if( _item > listbox->count()-2 )
       return;

    listbox->setAutoUpdate( FALSE );
    playlistMgr->moveDown( _item );
    listbox->setAutoUpdate( TRUE );
    listbox->repaint();
}

void Playlist::slotDelMPEG()
{
   if( listbox->currentItem() < 0 )
      return;
   
   playlistMgr->remove( listbox->currentItem() );
}

void Playlist::slotDrop( KDNDDropZone* _zone )
{
   QStrList strlist;
   KURL *url;
   char *str;
   
   strlist = _zone->getURLList();
   
   if ( strlist.count()>1 )
      for( str=strlist.first(); str!=0; str=strlist.next() )
      {
         url = new KURL( str );
         playlistMgr->insert( *url );
         delete url;
      }
   else
   {
      // Open first URL
      url = new KURL( strlist.first() );
      playlistMgr->insert( *url );
      delete url;
   }
}
